;+
;
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2005,
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;
;
; read_l1b_hdf
;
; PURPOSE:
;	Read TES L1B HDF product files (versions F02,F03)
;
; CALL:
; 	read_l1b_hdf,file,filter,spectra,nesr,freq_scale,status=status
;	start_frequency=start_frequency, delta_frequency=delta_frequency,
;	qa_data=qa_data,geo_data=geo_data,eng_data=eng_data,
;	l1b_meta=l1b_meta,tes_meta=tes_meta,hdfeos=hdfeos,
;	obs_view=obs_view,read_ph=read_ph,status=status
;	
;
; INPUTS:
;	file = name of file to be read (string)
; 	filter = name of filter subset to read e.g. '1A1' (string)
;
; KEYWORD INPUT:
;	read_ph = if set, read production history.  If not set, the
;		production history will be a null string
;
; OUTPUTS:
;	spectra = L1B spectra, LxMxN array (float), L spectral points,
;		M observations, N detectors
;	nesr = L1B nesr, LxMxN array (float), L spectral points,
;		M observations, N detectors
;		(note: spectral scale can be obtained with
;		read_l1b_hdf_spectra)
;	freq_scale = frequency scale (double array, L elements),
;		     which is dindgen(L)*delta_frequency+start_frequency
;
; KEYWORD OUTPUT:
;	start_frequency = attribute enumerating the start of the
;			  frequency grid for the spectra (double)
;	delta_frequency = attribute enumerating the grid spacing
;			  for the frequency grid of the spectra
;			  (double)
;
;	obs_view = contents of Observation_View filter swath attribute
;		(either 'Nadir' or 'Limb') for SO files,
;		'' (null string) otherwise
;		
;	qa_data = L1A and L1B quality information as array (number of
;		elements equals number of observations n) 
;		of structures of structures.  
;
;		The structures have the following fields:
;   L1A_PIXEL       STRUCT 
;   L1AQA           STRUCT
;   L1B_BT          STRUCT
;   L1B_TARGET      STRUCT 
;
; 	The sub-structures have the following fields:
;   L1A_PIXEL:
;   ----------
;   	L1A_DN_OVERFLOW_OCCURRED 	BYTE      Array[n,16]
;	L1A_DN_UNDERFLOW_OCCURRED	BYTE      Array[n,16]
;	L1A_SPIKE_OCCURRED		BYTE      Array[n,16]
;
;   L1AQA:
;   ------
;   L1A_BB_TEMPERATURE_OUT_OF_SPECIFICATION	BYTE      Array[n]
;   L1A_CHANNEL_SHIFT				BYTE      Array[n]
;   L1A_COLD_REF_PLATE_TEMPERATURE_OUT_OF_SPECIFICATION	BYTE      Array[n]
;   L1A_ENGINEERING_TEMPERATURES_OUT_OF_SPECIFICATION	BYTE      Array[n]
;   L1A_FILTER_SEQ_ERROR_OCCURRED		BYTE      Array[n]
;   L1A_ICS_DIRECTION_OUT_OF_SEQUENCE		BYTE      Array[n]
;   L1A_ICS_SPEED_VARIATION_OUT_OF_SPECIFICATION	BYTE      Array[n]
;   L1A_IFGM_DROPOUT_OCCURRED			BYTE      Array[n]
;   L1A_SC_ATTITUDE_OUT_OF_SPEC			BYTE      Array[n]
;   L1A_SEQUENCE_ERROR_OCCURRED			BYTE      Array[n]
;   L1A_TIME_DATE_ERROR_OCCURRED		BYTE      Array[n]
;
;   L1B_BT (for Nadir):
;   -------
;   L1B_N_INTERPIXEL_VARI_BT10		FLOAT     Array[n]
;   L1B_N_INTERPIXEL_VARI_BT11		FLOAT     Array[n]
;   L1B_N_IPIX_VAR_EXCEEDED_BT10	BYTE      Array[n]
;   L1B_N_IPIX_VAR_EXCEEDED_BT11	BYTE      Array[n]
;   L1B_NADIR_BT_10 			FLOAT     Array[n]
;   L1B_NADIR_BT_11 			FLOAT     Array[n]
;   L1B_NADIR_BT_12 			FLOAT     Array[n]
;   L1B_NADIR_BT_5  			FLOAT     Array[n]
;   L1B_NADIR_BT_8  			FLOAT     Array[n]
;
;   L1B_BT (for Limb):
;   -------
;   L1B_LIMB_BT_10 			FLOAT     Array[n,16]
;   L1B_LIMB_BT_11 			FLOAT     Array[n,16]
;   L1B_LIMB_BT_12 			FLOAT     Array[n,16]
;   L1B_LIMB_BT_5  			FLOAT     Array[n,16]
;   L1B_LIMB_BT_8  			FLOAT     Array[n,16]
;
;
;   L1B_TARGET:       
;   -----------
;   L1B_ABSOLUTE_RADIANCE_MEAN_BT	FLOAT     Array[n, 16]
;   L1B_ABSOLUTE_RADIANCE_MEAN_BT_FLAG  BYTE      Array[n, 16]
;   L1B_CAL_TABLE_QUALITY_FLAG		BYTE      Array[n, 16]
;   L1B_GENERAL_ERROR_FLAG		BYTE      Array[n, 16]
;   L1B_GENERAL_QUALITY_FLAG		BYTE      Array[n, 16]
;   L1B_IMAGINARY_MEAN			FLOAT     Array[n, 16]
;   L1B_IMAGINARY_MEAN_FLAG		BYTE      Array[n, 16]
;   L1B_IMAGINARY_RMS			FLOAT     Array[n, 16]
;   L1B_IMAGINARY_RMS_FLAG		BYTE      Array[n, 16]
;   L1B_MISSING_CAL_TABLE_FLAG		BYTE      Array[n, 16]
;   L1B_NESR_MEAN   			FLOAT     Array[n, 16]
;   L1B_NESR_MEAN_FLAG			BYTE      Array[n, 16]
;   L1B_PHASE_ALIGNMENT			FLOAT     Array[n, 16]
;   L1B_PHASE_ALIGNMENT_FLAG		BYTE      Array[n, 16]
;   L1B_ZERO_PADDING			FLOAT     Array[n, 16]
;   L1B_ZERO_PADDING_FLAG		BYTE      Array[n, 16]
;
;	geo_data = L1B Geolocation data, an
;		array (number of elements equals number of
;		observations) of structures.  The structures have
;		the following fields:
;
;   TIME_OF_GEOLOCATION		DOUBLE
;   GEOLOCATION_FAILED		BYTE   
;   ORBITASCENDINGFLAG		BYTE    
;   PATH_NUMBER    		BYTE     
;   PCS_QA_POOR_PERCENTAGE	BYTE       
;   DPREP_QA_POOR_PERCENTAGE	BYTE       
;   ORBIT_INCLINATION_ANGLE	DOUBLE        
;   LATITUDE        		DOUBLE        
;   LONGITUDE       		DOUBLE       
;   ELEVATION       		FLOAT         
;   HORIZONTAL_UNCERTAINTY	FLOAT        
;   ELEVATION_UNCERTAINTY	FLOAT        
;   LATITUDE_FOOTPRINT_1	DOUBLE        
;   LATITUDE_FOOTPRINT_2	DOUBLE       
;   LATITUDE_FOOTPRINT_3	DOUBLE       
;   LATITUDE_FOOTPRINT_4	DOUBLE      
;   LONGITUDE_FOOTPRINT_1	DOUBLE        
;   LONGITUDE_FOOTPRINT_2	DOUBLE        
;   LONGITUDE_FOOTPRINT_3	DOUBLE       
;   LONGITUDE_FOOTPRINT_4	DOUBLE         
;   SURFACEELEVATION		FLOAT        
;   SURFACEELEVSTANDARDDEVIATION FLOAT      
;   MIN_ELEV_FOOTPRINT		FLOAT                     
;   MAX_ELEV_FOOTPRINT		FLOAT     
;   SURFACE_TYPE_FOOTPRINT	BYTE   
;   DAY_NIGHT_FLAG_TARGET	BYTE   
;   DAY_NIGHT_FLAG_SC		BYTE 
;   LOCALSOLARTIME  		FLOAT       
;   TES_BSIGHT_AZIMUTH		DOUBLE         
;   TES_BSIGHT_AZIMUTH_UNCERT	DOUBLE     
;   TESBORESIGHTNADIRANGLE	DOUBLE    
;   TES_BSIGHT_NADIR_UNCERT	DOUBLE    
;   GRD_TRK_TES_BSIGHT_AZIMUTH	DOUBLE     
;   SPACECRAFTAZIMUTH		DOUBLE              
;   SPACECRAFTZENITH		DOUBLE   
;   TGT_SUN_AZIMUTH 		DOUBLE    
;   TGT_SUN_ZENITH_ANGLE	DOUBLE   
;   SOLARAZIMUTHANGLE		DOUBLE  
;   SOLARZENITHANGLE		DOUBLE   
;   M1_MIRROR_SUN_ANGLE		DOUBLE    
;   SPACECRAFTLATITUDE		DOUBLE    
;   SPACECRAFTLONGITUDE		DOUBLE   
;   SPACECRAFTALTITUDE		DOUBLE   
;   DOPPLER_SHIFT   		DOUBLE  
;   RESOLVED_SC_POSITION_X	DOUBLE    
;   RESOLVED_SC_POSITION_Y	DOUBLE    
;   RESOLVED_SC_POSITION_Z	DOUBLE   
;   RESOLVED_SC_VELOCITY_X	DOUBLE  
;   RESOLVED_SC_VELOCITY_Y	DOUBLE  
;   RESOLVED_SC_VELOCITY_Z	DOUBLE  
;   RESOLVED_SC_QUATERNION_Q1	DOUBLE  
;   RESOLVED_SC_QUATERNION_Q2	DOUBLE 
;   RESOLVED_SC_QUATERNION_Q3	DOUBLE 
;   RESOLVED_SC_QUATERNION_Q4	DOUBLE  
;   RESOLVED_SC_ATTITUDE_PITCH	DOUBLE  
;   RESOLVED_SC_ATTITUDE_ROLL	DOUBLE       
;   RESOLVED_SC_ATTITUDE_YAW	DOUBLE 
;   RESOLVED_SC_PITCH_RATE	DOUBLE    
;   RESOLVED_SC_ROLL_RATE	DOUBLE    
;   RESOLVED_SC_YAW_RATE	DOUBLE   
;   PCS_TRACK       		DOUBLE   
;   PCS_CROSSTRACK  		DOUBLE  
;
;	eng_data = L1B engineering data from L1A diagnostic file, an
;		array (number of elements equals number of
;		observations) of structures.  The structures have
;		the following fields:
;
; 		SCAN            	BYTE         
; 		SEQUENCE        	INT              
; 		TIME_OF_OBSERVATION  	DOUBLE      
;    		FILTER_IDENTIFICATION   STRING
;    		ADC_ENABLED     	INT     
;    		ELECT_FILTER    	FLOAT          
;    		FRINGE_CLOCK_DIVISOR	INT        
;    		AT_INDEX       		BYTE 
;    		FILTER_WHEEL_INDEX	INT 
;    		SIGNAL_CHAIN_GAIN	FLOAT
;    		DETECTOR_TEMP   	FLOAT 
;    		OBSERVATION_TYPE	INT   
;    		ICS_SCAN_DIRECTION	STRING
;    		BLACKBODY_WALL_TEMP__1	FLOAT    
;    		BLACKBODY_WALL_TEMP__2	FLOAT 
;    		COLD_REFERENCE_PLATE_TEMP__1	FLOAT
;    		COLD_REFERENCE_PLATE_TEMP__2	FLOAT
;    		BEAMSPLITTER_TEMP__1	FLOAT
;    		BEAMSPLITTER_TEMP__2	FLOAT
;    		FOREOPTICS_TEMP 	FLOAT       
;    		M1_MIRROR_TEMP  	FLOAT 
;    		M2_MIRROR_TEMP  	FLOAT   
;    		CALIBRATION_SS_RESISTOR_1	FLOAT
;    		CALIBRATION_SS_RESISTOR_2	FLOAT
;    		OSE_RESISTOR_1  	FLOAT
;    		OSE_RESISTOR_2  	FLOAT
;
;
;	l1b_meta = metadata information as a structure
;	with the following fields:
;	       run_counter  (long)
;	       orbital_path_id (byte)
;	       absolute_orbit_number (long)
;	       time_of_observation_start (double)
;	       time_of_observation_end (double)
;	       geolocation_data_missing (integer)
;	       icecontamination (byte)
;	       l1a_data_missing (double)
;	       l1b_data_missing (double)
;	       command_seq_id (long) (F02) (zero for F03)
;
;	tes_meta = metadata information as a structure
;	with the following fields:
;	       productionhistory (string)
;	       instrumentname (string)
;	       processlevel (string)
;	       TAI93at0zOfGranule (double)
;	       GlobalSurveyNumber (long64)
;	       granulemonth (byte)
;	       granuleday (byte)
;	       granuleyear (integer)
;	       surveymode (string)
;	       pgeversion (string)
;	       Command_Seq_ID  (long) (F03) (zero for F02)
;
;	hdfeos = coremetadata information (a hugh string)
;
;	status = 0 for success
;		 1 if file does not exist
;		 2 the file name does not have expected version numbres
;		 3 the file is not HDF 5
;		 4 Unkown file type (not nadir or limb)
; 
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 07/2005
;
; UPDATE HISTORY:
;	Brendan Fisher, JPL, 12/2005
;	-adapted from individual routines to read complete file (for a 
;	 filter)
;	-updated to work with version F03_01 files
; 	-changed TAI93atOzOfGranule to TAI93at0zOfGranule for tes_meta
; 	 output of all versions
;	Brendan Fisher, JPL, 01/2006
;		-added limb functionality
;		-added new version check
;	Brendan Fisher, JPL, 07/2006 fixed bug related to reading
;		limb special observations files.  Added obs_view keyword
; 
;-
pro read_l1b_hdf,file,filter,spectra,nesr,freq_scale, $
                 start_frequency=start_frequency, $
                 delta_frequency=delta_frequency, $
                 qa_data=qa_data,geo_data=geo_data, $
                 eng_data=eng_data,l1b_meta=l1b_meta, $
                 tes_meta=tes_meta,hdfeos=hdfeos, $ $
                 obs_view=obs_view,$
                 status=status,read_ph=read_ph
if n_params() ne 5 then begin
    print,'Usage:'
    print,'read_l1b_hdf,file,filter,spectra,nesr,freq_scale, $'
    print,'start_frequency = start_frequency, $'
    print,'delta_frequency = delta_frequency, $'
    print,'qa_data=qa_data,geo_data=geo_data, $'
    print,'eng_data=eng_data,l1b_meta=l1b_meta, $'
    print,'tes_meta=tes_meta,hdfeos=hdfeos, $'
    print,'obs_view=obs_view, $'
    print,'status=status'
    return
endif


status=0
if n_elements(read_ph) eq 0 then read_ph = 0

; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return
endif
; check version
pos2 = strpos(file,'_F02_')
pos3 = strpos(file,'_F03_')
;pos4 = strpos(file,'F04_01')
if pos2 ne -1 then version = 2
if pos3 ne -1 then version = 3
;if pos4 ne -1 then version = 4

if pos2 eq -1 and  pos3 eq -1 then begin
    print,'Expected version _F02_, or _F03_ in file name not found.'
    print,file
    status = 2
    return
endif
; check to see if file is an HDF5 
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return
endif
;
; determine file type
;
npos1 = strpos(file,'Nadir')
npos2 = strpos(file,'SO-Low')
lpos1 = strpos(file,'Limb')
lpos2 = strpos(file,'SO-High')
file_type = ''
if npos1 ne -1 or npos2 ne -1 then file_type = 'nadir'
if lpos1 ne -1 or lpos2 ne -1 then file_type = 'limb'
if file_type eq '' then begin
    print,'Unable to determine if file is Nadir or Limb type.'
    print,file
    status=4
    return
endif
    

fid = h5f_open(file)

groupid = h5g_open(fid,'Filter'+strupcase(filter))

; spectra
specid = h5d_open(groupid,'Spectra')
spectra = h5d_read(specid)
h5d_close,specid

freqid = h5a_open_name(groupid,'Start_Frequency')
start_frequency = h5a_read(freqid)
delnuid = h5a_open_name(groupid,'Delta_Frequency')
delta_frequency = h5a_read(delnuid)

h5a_close,freqid
h5a_close,delnuid

obs_view = ''
ppos = strpos(file,'-SO-')
if ppos ne -1 then begin
    if version ge 3 then begin
        obsid = h5a_open_name(groupid,'Observation_view')
        obs_view= h5a_read(obsid)
        h5a_close,obsid
    endif else obs_view = ''
    file_type = strlowcase(obs_view)
endif

; nesr
id = h5d_open(groupid,'NESR')

nesr = h5d_read(id)

h5d_close,id


; Read a QA field to get the dimensions
qaid = h5g_open(groupid,'QA')

l1aqa = h5g_open(qaid,'L1AQA')

id = h5d_open(l1aqa,'L1A_BB_Temperature_Out_Of_Specification')
temp = h5d_read(id)
h5d_close,id

h5g_close,l1aqa

; get number of elements for all fields
nn = n_elements(temp) 
; create a structure of structures to hold all of the QA data

l1a_pixel = { $
            l1a_dn_overflow_occurred : bytarr(nn,16), $
            l1a_dn_underflow_occurred :  bytarr(nn,16), $
            l1a_spike_occurred :  bytarr(nn,16) }
l1aqa = { $
        l1a_bb_temperature_out_of_specification : bytarr(nn), $
        l1a_channel_shift : bytarr(nn), $
        l1a_cold_ref_plate_temperature_out_of_specification : bytarr(nn), $
        l1a_engineering_temperatures_out_of_specification : bytarr(nn), $
        l1a_filter_seq_error_occurred : bytarr(nn), $
        l1a_ics_direction_out_of_sequence : bytarr(nn), $
        l1a_ics_speed_variation_out_of_specification : bytarr(nn), $
        l1a_ifgm_dropout_occurred : bytarr(nn), $
        l1a_sc_attitude_out_of_spec : bytarr(nn), $
        l1a_sequence_error_occurred : bytarr(nn), $
        l1a_time_date_error_occurred : bytarr(nn) }

if file_type eq 'nadir' then begin
    l1b_bt = { $
             l1b_n_interpixel_vari_bt10 : fltarr(nn), $
             l1b_n_interpixel_vari_bt11 : fltarr(nn), $
             l1b_n_ipix_var_exceeded_bt10 : bytarr(nn), $
             l1b_n_ipix_var_exceeded_bt11 : bytarr(nn), $
             l1b_nadir_bt_10 : fltarr(nn), $
             l1b_nadir_bt_11 : fltarr(nn), $
             l1b_nadir_bt_12 : fltarr(nn), $
             l1b_nadir_bt_5 : fltarr(nn), $
             l1b_nadir_bt_8 : fltarr(nn) }
endif
if file_type eq 'limb' then begin

    l1b_bt = { $
             l1b_limb_bt_10 : fltarr(nn,16), $
             l1b_limb_bt_11 : fltarr(nn,16), $
             l1b_limb_bt_12 : fltarr(nn,16), $
             l1b_limb_bt_5 : fltarr(nn,16), $
             l1b_limb_bt_8 : fltarr(nn,16) }
endif

l1b_target = { $
             l1b_absolute_radiance_mean_bt : fltarr(nn,16), $
             l1b_absolute_radiance_mean_bt_flag : bytarr(nn,16), $
             l1b_cal_table_quality_flag : bytarr(nn,16), $
             l1b_general_error_flag : bytarr(nn,16), $
             l1b_general_quality_flag : bytarr(nn,16), $
             l1b_imaginary_mean : fltarr(nn,16), $
             l1b_imaginary_mean_flag : bytarr(nn,16), $
             l1b_imaginary_rms : fltarr(nn,16), $
             l1b_imaginary_rms_flag : bytarr(nn,16), $
             l1b_missing_cal_table_flag : bytarr(nn,16), $
             l1b_nesr_mean : fltarr(nn,16), $
             l1b_nesr_mean_flag : bytarr(nn,16), $
             l1b_phase_alignment : fltarr(nn,16), $
             l1b_phase_alignment_flag : bytarr(nn,16), $
             l1b_zero_padding : fltarr(nn,16), $
             l1b_zero_padding_flag : bytarr(nn,16) }



         
;------------------------------------------------      
; L1A Pixel QA
l1apid = h5g_open(qaid,'L1APixelQA')

id = h5d_open(l1apid,'L1A_DN_Overflow_Occurred')
l1a_pixel.L1A_DN_Overflow_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1apid,'L1A_DN_Underflow_Occurred')
l1a_pixel.L1A_DN_Underflow_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1apid,'L1A_Spike_Occurred')
l1a_pixel.L1A_Spike_Occurred = h5d_read(id)
h5d_close,id

h5g_close,l1apid


;------------------------------------------------      
; L1A QA
l1aqaid = h5g_open(qaid,'L1AQA')

id = h5d_open(l1aqaid,'L1A_BB_Temperature_Out_Of_Specification')
l1aqa.L1A_BB_Temperature_Out_Of_Specification = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Channel_Shift')
l1aqa.L1A_Channel_Shift = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Cold_Ref_Plate_Temperature_Out_Of_Specification')
l1aqa.L1A_Cold_Ref_Plate_Temperature_Out_Of_Specification = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Engineering_Temperatures_Out_Of_Specification')
l1aqa.L1A_Engineering_Temperatures_Out_Of_Specification = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Filter_Seq_Error_Occurred')
l1aqa.L1A_Filter_Seq_Error_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_ICS_Direction_Out_Of_Sequence')
l1aqa.L1A_ICS_Direction_Out_Of_Sequence = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_ICS_Speed_Variation_Out_Of_Specification')
l1aqa.L1A_ICS_Speed_Variation_Out_Of_Specification = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Ifgm_Dropout_Occurred')
l1aqa.L1A_Ifgm_Dropout_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_SC_Attitude_Out_Of_Spec')
l1aqa.L1A_SC_Attitude_Out_Of_Spec = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Sequence_Error_Occurred')
l1aqa.L1A_Sequence_Error_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Time_Date_Error_Occurred')
l1aqa.L1A_Time_Date_Error_Occurred = h5d_read(id)
h5d_close,id

h5g_close,l1aqaid

;------------------------------------------------      
; L1B BT

l1bbtid = h5g_open(qaid,'L1B_Brightness_Temperatures')

if file_type eq 'nadir' then begin
    id = h5d_open(l1bbtid,'L1B_N_InterPixel_Vari_BT10')
    l1b_bt.L1B_N_InterPixel_Vari_BT10 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_N_InterPixel_Vari_BT11')
    l1b_bt.L1B_N_InterPixel_Vari_BT11 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_N_Ipix_Var_Exceeded_BT10')
    l1b_bt.L1B_N_Ipix_Var_Exceeded_BT10 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_N_Ipix_Var_Exceeded_BT11')
    l1b_bt.L1B_N_Ipix_Var_Exceeded_BT11 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Nadir_BT_10')
    l1b_bt.L1B_Nadir_BT_10 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Nadir_BT_11')
    l1b_bt.L1B_Nadir_BT_11 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Nadir_BT_12')
    l1b_bt.L1B_Nadir_BT_12 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Nadir_BT_5')
    l1b_bt.L1B_Nadir_BT_5 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Nadir_BT_8')
    l1b_bt.L1B_Nadir_BT_8 = h5d_read(id)
    h5d_close,id
endif
if file_type eq 'limb' then begin
    id = h5d_open(l1bbtid,'L1B_Limb_BT_10')
    l1b_bt.L1B_Limb_BT_10 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Limb_BT_11')
    l1b_bt.L1B_Limb_BT_11 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Limb_BT_12')
    l1b_bt.L1B_Limb_BT_12 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Limb_BT_5')
    l1b_bt.L1B_Limb_BT_5 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Limb_BT_8')
    l1b_bt.L1B_Limb_BT_8 = h5d_read(id)
    h5d_close,id
endif

h5g_close,l1bbtid

;------------------------------------------------      
; L1B BT
l1btgtid = h5g_open(qaid,'L1B_Target_Spectra')

id = h5d_open(l1btgtid,'L1B_Absolute_Radiance_Mean_BT')
l1b_target.L1B_Absolute_Radiance_Mean_BT = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Absolute_Radiance_Mean_BT_Flag')
l1b_target.L1B_Absolute_Radiance_Mean_BT_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Cal_Table_Quality_Flag')
l1b_target.L1B_Cal_Table_Quality_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_General_Error_Flag')
l1b_target.L1B_General_Error_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_General_Quality_Flag')
l1b_target.L1B_General_Quality_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Imaginary_Mean')
l1b_target.L1B_Imaginary_Mean = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Imaginary_Mean_Flag')
l1b_target.L1B_Imaginary_Mean_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Imaginary_RMS')
l1b_target.L1B_Imaginary_RMS = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Imaginary_RMS_Flag')
l1b_target.L1B_Imaginary_RMS_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Missing_Cal_Table_Flag')
l1b_target.L1B_Missing_Cal_Table_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_NESR_Mean')
l1b_target.L1B_NESR_Mean = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_NESR_Mean_Flag')
l1b_target.L1B_NESR_Mean_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Phase_Alignment')
l1b_target.L1B_Phase_Alignment = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Phase_Alignment_Flag')
l1b_target.L1B_Phase_Alignment_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Phase_Alignment_Flag')
l1b_target.L1B_Phase_Alignment_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Zero_Padding_Flag')
l1b_target.L1B_Zero_Padding_Flag = h5d_read(id)
h5d_close,id

h5g_close,l1btgtid



qa_data = { $
     l1a_pixel : l1a_pixel, $
     l1aqa : l1aqa, $
     l1b_bt : l1b_bt, $
     l1b_target : l1b_target }
            

; geolocation data
id = h5d_open(groupid,'Geolocation')

geo_data = h5d_read(id)

h5d_close,id

; engineering data
engid = h5d_open(groupid,'Engineering')

eng_data = h5d_read(engid)

h5d_close,engid

; L1B Metadata
subid = h5g_open(fid,'L1B_MetadataGroup')

data = { $
       run_counter : 0l, $
       orbital_path_id : 0b, $
       absolute_orbit_number : 0l, $
       time_of_observation_start : 0.0d, $
       time_of_observation_end : 0.0d, $
       geolocation_data_missing : 0, $
       icecontamination : 0b, $
       l1a_data_missing : 0.0, $
       l1b_data_missing : 0.0, $
       command_seq_id : 0l }

id = h5a_open_name(subid,'Run_Counter')
data.run_counter = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'Orbital_Path_ID')
data.orbital_path_id = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'Time_Of_Observation_Start')
data.time_of_observation_start = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'Time_Of_Observation_End')
data.time_of_observation_end = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'Geolocation_Data_Missing')
data.geolocation_data_missing = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'IceContamination')
data.icecontamination = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'L1A_Data_Missing')
data.l1a_data_missing = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'L1B_Data_Missing')
data.l1b_data_missing = h5a_read(id)
h5a_close,id

if version eq 2 then begin
    id = h5a_open_name(subid,'Command_Seq_ID')
    data.command_seq_id = h5a_read(id)
    h5a_close,id
endif else data.command_seq_id = 0

l1b_meta = temporary(data)

h5g_close,subid


; TES metadata
subid = h5g_open(fid,'TESMetadataGroup')


data = { $
       productionhistory : '', $
       instrumentname : '', $
       processlevel : '', $
       TAI93at0zOfGranule : 0.0d, $
       GlobalSurveyNumber : 0ll, $
       granulemonth : 0b, $
       granuleday : 0b, $
       granuleyear : 0, $
       surveymode : '', $
       pgeversion : '', $
       command_seq_id : 0l $
}

if read_ph ne 0 then begin
    id = h5d_open(subid,'ProductionHistory')
    data.productionhistory = h5d_read(id)
    h5d_close,id
endif else ProductionHistory = ''

id = h5a_open_name(subid,'InstrumentName')
data.instrumentname = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'ProcessLevel')
data.processlevel = h5a_read(id)
h5a_close,id

if version eq 2 then begin
    id = h5a_open_name(subid,'TAI93AtOzOfGranule')
    TAI93AtOzOfGranule = h5a_read(id)
    h5a_close,id
endif
if version ge 3 then begin
    id = h5a_open_name(subid,'TAI93At0zOfGranule')
    TAI93At0zOfGranule = h5a_read(id)
    h5a_close,id
endif    

id = h5a_open_name(subid,'GlobalSurveyNumber')
data.GlobalSurveyNumber = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'GranuleMonth')
data.granulemonth = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'GranuleDay')
data.granuleday = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'GranuleYear')
data.granuleyear = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'SurveyMode')
data.surveymode = h5a_read(id)
h5a_close,id

id = h5a_open_name(subid,'PGEVersion')
data.pgeversion = h5a_read(id)
h5a_close,id

if version ge 3 then begin
    id = h5a_open_name(subid,'Command_Seq_ID')
    data.command_seq_id = h5a_read(id)
    h5a_close,id
endif else data.command_seq_id = 0

tes_meta = temporary(data)

h5g_close,subid



hdfid = h5g_open(fid,'HDFEOS INFORMATION')

coreid = h5d_open(hdfid,'coremetadata')
hdfoes = h5d_read(coreid)
h5d_close,coreid

h5g_close,hdfid




h5g_close, groupid
h5f_close,fid

; make frequency scale
s = size(spectra)
freq_scale=dindgen(s[1])*delta_frequency+start_frequency


return

end
