;+
;
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2005,
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;
;
; read_l1b_hdf_qa
;
; PURPOSE:
;	Read TES L1B Engineering data from L1B HDF product files
;	(version F02, F03)
;
; CALL:
;	 read_l1b_hdf_qa,file,filter,qa_data,status=status
;
; INPUTS:
;	file = name of file to be read (string)
; 	filter = name of filter subset to read (string)
;
; OUTPUTS:
;	data = L1A and L1B quality information as array (number of
;		elements equals number of observations n) 
;		of structures of structures.  
;
;		The structures have the following fields:
;   L1A_PIXEL       STRUCT 
;   L1AQA           STRUCT
;   L1B_BT          STRUCT
;   L1B_TARGET      STRUCT 
;
; 	The sub-structures have the following fields:
;   L1A_PIXEL:
;   ----------
;   	L1A_DN_OVERFLOW_OCCURRED 	BYTE      Array[n,16]
;	L1A_DN_UNDERFLOW_OCCURRED	BYTE      Array[n,16]
;	L1A_SPIKE_OCCURRED		BYTE      Array[n,16]
;
;   L1AQA:
;   ------
;   L1A_BB_TEMPERATURE_OUT_OF_SPECIFICATION	BYTE      Array[n]
;   L1A_CHANNEL_SHIFT				BYTE      Array[n]
;   L1A_COLD_REF_PLATE_TEMPERATURE_OUT_OF_SPECIFICATION	BYTE      Array[n]
;   L1A_ENGINEERING_TEMPERATURES_OUT_OF_SPECIFICATION	BYTE      Array[n]
;   L1A_FILTER_SEQ_ERROR_OCCURRED		BYTE      Array[n]
;   L1A_ICS_DIRECTION_OUT_OF_SEQUENCE		BYTE      Array[n]
;   L1A_ICS_SPEED_VARIATION_OUT_OF_SPECIFICATION	BYTE      Array[n]
;   L1A_IFGM_DROPOUT_OCCURRED			BYTE      Array[n]
;   L1A_SC_ATTITUDE_OUT_OF_SPEC			BYTE      Array[n]
;   L1A_SEQUENCE_ERROR_OCCURRED			BYTE      Array[n]
;   L1A_TIME_DATE_ERROR_OCCURRED		BYTE      Array[n]
;
;   L1B_BT (for Nadir):
;   -------
;   L1B_N_INTERPIXEL_VARI_BT10		FLOAT     Array[n]
;   L1B_N_INTERPIXEL_VARI_BT11		FLOAT     Array[n]
;   L1B_N_IPIX_VAR_EXCEEDED_BT10	BYTE      Array[n]
;   L1B_N_IPIX_VAR_EXCEEDED_BT11	BYTE      Array[n]
;   L1B_NADIR_BT_10 			FLOAT     Array[n]
;   L1B_NADIR_BT_11 			FLOAT     Array[n]
;   L1B_NADIR_BT_12 			FLOAT     Array[n]
;   L1B_NADIR_BT_5  			FLOAT     Array[n]
;   L1B_NADIR_BT_8  			FLOAT     Array[n]
;
;   L1B_BT (for Limb):
;   -------
;   L1B_LIMB_BT_10 			FLOAT     Array[n,16]
;   L1B_LIMB_BT_11 			FLOAT     Array[n,16]
;   L1B_LIMB_BT_12 			FLOAT     Array[n,16]
;   L1B_LIMB_BT_5  			FLOAT     Array[n,16]
;   L1B_LIMB_BT_8  			FLOAT     Array[n,16]
;
;   L1B_TARGET:       
;   -----------
;   L1B_ABSOLUTE_RADIANCE_MEAN_BT	FLOAT     Array[n, 16]
;   L1B_ABSOLUTE_RADIANCE_MEAN_BT_FLAG  BYTE      Array[n, 16]
;   L1B_CAL_TABLE_QUALITY_FLAG		BYTE      Array[n, 16]
;   L1B_GENERAL_ERROR_FLAG		BYTE      Array[n, 16]
;   L1B_GENERAL_QUALITY_FLAG		BYTE      Array[n, 16]
;   L1B_IMAGINARY_MEAN			FLOAT     Array[n, 16]
;   L1B_IMAGINARY_MEAN_FLAG		BYTE      Array[n, 16]
;   L1B_IMAGINARY_RMS			FLOAT     Array[n, 16]
;   L1B_IMAGINARY_RMS_FLAG		BYTE      Array[n, 16]
;   L1B_MISSING_CAL_TABLE_FLAG		BYTE      Array[n, 16]
;   L1B_NESR_MEAN   			FLOAT     Array[n, 16]
;   L1B_NESR_MEAN_FLAG			BYTE      Array[n, 16]
;   L1B_PHASE_ALIGNMENT			FLOAT     Array[n 16]
;   L1B_PHASE_ALIGNMENT_FLAG		BYTE      Array[n 16]
;   L1B_ZERO_PADDING			FLOAT     Array[n 16]
;   L1B_ZERO_PADDING_FLAG		BYTE      Array[n 16]
;
;
; KEYWORD OUTPUT:
;	status = 0 for success
;		 1 if file does not exist
;		 2 the file name does not have F02_01 in name
;		 3 the file is not HDF 5
; 
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 07/2005
;
; UPDATE HISTORY:
;	Brendan Fisher, JPL, 01/2006
;		-added limb functionality
;		-added new version check
;	Brendan Fisher, JPL, 07/2006 fixed bug related to reading
;		limb special observations files.
;
;-
pro read_l1b_hdf_qa,file,filter,qa,status=status

if n_params() ne 3 then begin
    print,'Usage:'
    print,'read_l1b_hdf_qa,file,filter,qa_data,status=status'
    return
endif

status=0
; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return
endif
; check version
pos2 = strpos(file,'_F02_')
pos3 = strpos(file,'_F03_')
if pos2 ne -1 then version = 2
if pos3 ne -1 then version = 3
pos=0
if pos eq -1 then begin
    print,'Expected version _F02_ or _F03_ in file name not found.'
    print,file
    status = 2
    return
endif
; check to see if file is an HDF5 
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return
endif

;
; determine file type
;
npos1 = strpos(file,'Nadir')
npos2 = strpos(file,'SO-Low')
lpos1 = strpos(file,'Limb')
lpos2 = strpos(file,'SO-High')
file_type = ''
if npos1 ne -1 or npos2 ne -1 then file_type = 'nadir'
if lpos1 ne -1 or lpos2 ne -1 then file_type = 'limb'
if file_type eq '' then begin
    print,'Unable to determine if file is Nadir or Limb type.'
    print,file
    status=4
    return
endif


fid = h5f_open(file)

groupid = h5g_open(fid,'Filter'+strupcase(filter))

ppos = strpos(file,'-SO-')
if ppos ne -1 then begin
    if version ge 3 then begin
        obsid = h5a_open_name(groupid,'Observation_view')
        obs_view= h5a_read(obsid)
        h5a_close,obsid
    endif else obs_view = ''
    file_type = strlowcase(obs_view)
endif

qaid = h5g_open(groupid,'QA')

l1aqa = h5g_open(qaid,'L1AQA')

id = h5d_open(l1aqa,'L1A_BB_Temperature_Out_Of_Specification')
temp = h5d_read(id)
h5d_close,id

h5g_close,l1aqa

; get number of elements for all fields
nn = n_elements(temp) 
; create a structure of structures to hold all of the QA data

l1a_pixel = { $
            l1a_dn_overflow_occurred : bytarr(nn,16), $
            l1a_dn_underflow_occurred :  bytarr(nn,16), $
            l1a_spike_occurred :  bytarr(nn,16) }
l1aqa = { $
        l1a_bb_temperature_out_of_specification : bytarr(nn), $
        l1a_channel_shift : bytarr(nn), $
        l1a_cold_ref_plate_temperature_out_of_specification : bytarr(nn), $
        l1a_engineering_temperatures_out_of_specification : bytarr(nn), $
        l1a_filter_seq_error_occurred : bytarr(nn), $
        l1a_ics_direction_out_of_sequence : bytarr(nn), $
        l1a_ics_speed_variation_out_of_specification : bytarr(nn), $
        l1a_ifgm_dropout_occurred : bytarr(nn), $
        l1a_sc_attitude_out_of_spec : bytarr(nn), $
        l1a_sequence_error_occurred : bytarr(nn), $
        l1a_time_date_error_occurred : bytarr(nn) }

if file_type eq 'nadir' then begin
    l1b_bt = { $
             l1b_n_interpixel_vari_bt10 : fltarr(nn), $
             l1b_n_interpixel_vari_bt11 : fltarr(nn), $
             l1b_n_ipix_var_exceeded_bt10 : bytarr(nn), $
             l1b_n_ipix_var_exceeded_bt11 : bytarr(nn), $
             l1b_nadir_bt_10 : fltarr(nn), $
             l1b_nadir_bt_11 : fltarr(nn), $
             l1b_nadir_bt_12 : fltarr(nn), $
             l1b_nadir_bt_5 : fltarr(nn), $
             l1b_nadir_bt_8 : fltarr(nn) }
endif
if file_type eq 'limb' then begin

    l1b_bt = { $
             l1b_limb_bt_10 : fltarr(nn,16), $
             l1b_limb_bt_11 : fltarr(nn,16), $
             l1b_limb_bt_12 : fltarr(nn,16), $
             l1b_limb_bt_5 : fltarr(nn,16), $
             l1b_limb_bt_8 : fltarr(nn,16) }
endif

l1b_target = { $
             l1b_absolute_radiance_mean_bt : fltarr(nn,16), $
             l1b_absolute_radiance_mean_bt_flag : bytarr(nn,16), $
             l1b_cal_table_quality_flag : bytarr(nn,16), $
             l1b_general_error_flag : bytarr(nn,16), $
             l1b_general_quality_flag : bytarr(nn,16), $
             l1b_imaginary_mean : fltarr(nn,16), $
             l1b_imaginary_mean_flag : bytarr(nn,16), $
             l1b_imaginary_rms : fltarr(nn,16), $
             l1b_imaginary_rms_flag : bytarr(nn,16), $
             l1b_missing_cal_table_flag : bytarr(nn,16), $
             l1b_nesr_mean : fltarr(nn,16), $
             l1b_nesr_mean_flag : bytarr(nn,16), $
             l1b_phase_alignment : fltarr(nn,16), $
             l1b_phase_alignment_flag : bytarr(nn,16), $
             l1b_zero_padding : fltarr(nn,16), $
             l1b_zero_padding_flag : bytarr(nn,16) }



         
;------------------------------------------------      
; L1A Pixel QA
l1apid = h5g_open(qaid,'L1APixelQA')

id = h5d_open(l1apid,'L1A_DN_Overflow_Occurred')
l1a_pixel.L1A_DN_Overflow_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1apid,'L1A_DN_Underflow_Occurred')
l1a_pixel.L1A_DN_Underflow_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1apid,'L1A_Spike_Occurred')
l1a_pixel.L1A_Spike_Occurred = h5d_read(id)
h5d_close,id

h5g_close,l1apid


;------------------------------------------------      
; L1A QA
l1aqaid = h5g_open(qaid,'L1AQA')

id = h5d_open(l1aqaid,'L1A_BB_Temperature_Out_Of_Specification')
l1aqa.L1A_BB_Temperature_Out_Of_Specification = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Channel_Shift')
l1aqa.L1A_Channel_Shift = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Cold_Ref_Plate_Temperature_Out_Of_Specification')
l1aqa.L1A_Cold_Ref_Plate_Temperature_Out_Of_Specification = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Engineering_Temperatures_Out_Of_Specification')
l1aqa.L1A_Engineering_Temperatures_Out_Of_Specification = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Filter_Seq_Error_Occurred')
l1aqa.L1A_Filter_Seq_Error_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_ICS_Direction_Out_Of_Sequence')
l1aqa.L1A_ICS_Direction_Out_Of_Sequence = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_ICS_Speed_Variation_Out_Of_Specification')
l1aqa.L1A_ICS_Speed_Variation_Out_Of_Specification = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Ifgm_Dropout_Occurred')
l1aqa.L1A_Ifgm_Dropout_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_SC_Attitude_Out_Of_Spec')
l1aqa.L1A_SC_Attitude_Out_Of_Spec = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Sequence_Error_Occurred')
l1aqa.L1A_Sequence_Error_Occurred = h5d_read(id)
h5d_close,id

id = h5d_open(l1aqaid,'L1A_Time_Date_Error_Occurred')
l1aqa.L1A_Time_Date_Error_Occurred = h5d_read(id)
h5d_close,id

h5g_close,l1aqaid

;------------------------------------------------      
; L1B BT
l1bbtid = h5g_open(qaid,'L1B_Brightness_Temperatures')

if file_type eq 'nadir' then begin
    id = h5d_open(l1bbtid,'L1B_N_InterPixel_Vari_BT10')
    l1b_bt.L1B_N_InterPixel_Vari_BT10 = h5d_read(id)
    h5d_close,id

    id = h5d_open(l1bbtid,'L1B_N_InterPixel_Vari_BT11')
    l1b_bt.L1B_N_InterPixel_Vari_BT11 = h5d_read(id)
    h5d_close,id

    id = h5d_open(l1bbtid,'L1B_N_Ipix_Var_Exceeded_BT10')
    l1b_bt.L1B_N_Ipix_Var_Exceeded_BT10 = h5d_read(id)
    h5d_close,id

    id = h5d_open(l1bbtid,'L1B_N_Ipix_Var_Exceeded_BT11')
    l1b_bt.L1B_N_Ipix_Var_Exceeded_BT11 = h5d_read(id)
    h5d_close,id

    id = h5d_open(l1bbtid,'L1B_Nadir_BT_10')
    l1b_bt.L1B_Nadir_BT_10 = h5d_read(id)
    h5d_close,id

    id = h5d_open(l1bbtid,'L1B_Nadir_BT_11')
    l1b_bt.L1B_Nadir_BT_11 = h5d_read(id)
    h5d_close,id

    id = h5d_open(l1bbtid,'L1B_Nadir_BT_12')
    l1b_bt.L1B_Nadir_BT_12 = h5d_read(id)
    h5d_close,id

    id = h5d_open(l1bbtid,'L1B_Nadir_BT_5')
    l1b_bt.L1B_Nadir_BT_5 = h5d_read(id)
    h5d_close,id

    id = h5d_open(l1bbtid,'L1B_Nadir_BT_8')
    l1b_bt.L1B_Nadir_BT_8 = h5d_read(id)
    h5d_close,id
endif
if file_type eq 'limb' then begin
    id = h5d_open(l1bbtid,'L1B_Limb_BT_10')
    l1b_bt.L1B_Limb_BT_10 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Limb_BT_11')
    l1b_bt.L1B_Limb_BT_11 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Limb_BT_12')
    l1b_bt.L1B_Limb_BT_12 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Limb_BT_5')
    l1b_bt.L1B_Limb_BT_5 = h5d_read(id)
    h5d_close,id
    
    id = h5d_open(l1bbtid,'L1B_Limb_BT_8')
    l1b_bt.L1B_Limb_BT_8 = h5d_read(id)
    h5d_close,id
endif

h5g_close,l1bbtid

;------------------------------------------------      
; L1B BT
l1btgtid = h5g_open(qaid,'L1B_Target_Spectra')

id = h5d_open(l1btgtid,'L1B_Absolute_Radiance_Mean_BT')
l1b_target.L1B_Absolute_Radiance_Mean_BT = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Absolute_Radiance_Mean_BT_Flag')
l1b_target.L1B_Absolute_Radiance_Mean_BT_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Cal_Table_Quality_Flag')
l1b_target.L1B_Cal_Table_Quality_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_General_Error_Flag')
l1b_target.L1B_General_Error_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_General_Quality_Flag')
l1b_target.L1B_General_Quality_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Imaginary_Mean')
l1b_target.L1B_Imaginary_Mean = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Imaginary_Mean_Flag')
l1b_target.L1B_Imaginary_Mean_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Imaginary_RMS')
l1b_target.L1B_Imaginary_RMS = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Imaginary_RMS_Flag')
l1b_target.L1B_Imaginary_RMS_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Missing_Cal_Table_Flag')
l1b_target.L1B_Missing_Cal_Table_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_NESR_Mean')
l1b_target.L1B_NESR_Mean = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_NESR_Mean_Flag')
l1b_target.L1B_NESR_Mean_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Phase_Alignment')
l1b_target.L1B_Phase_Alignment = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Phase_Alignment_Flag')
l1b_target.L1B_Phase_Alignment_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Phase_Alignment_Flag')
l1b_target.L1B_Phase_Alignment_Flag = h5d_read(id)
h5d_close,id

id = h5d_open(l1btgtid,'L1B_Zero_Padding_Flag')
l1b_target.L1B_Zero_Padding_Flag = h5d_read(id)
h5d_close,id

h5g_close,l1btgtid

h5g_close,qaid

h5g_close, groupid
h5f_close,fid


qa = { $
     l1a_pixel : l1a_pixel, $
     l1aqa : l1aqa, $
     l1b_bt : l1b_bt, $
     l1b_target : l1b_target }


return

end
