;+
;
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2005,
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;
;
; read_l1b_hdf_spectra;
;
; PURPOSE:
;	Read TES L1B spectra from L1B HDF product files (version F02, F03)
;
; CALL:
; 	read_l1b_hdf_spectra,file,filter,spectra,freq_scale,status=status
;	start_frequency=start_frequency, delta_frequency=delta_frequency
;
; INPUTS:
;	file = name of file to be read (string)
; 	filter = name of filter subset to read (string)
;
; OUTPUTS:
;	spectra = L1B spectra, LxMxN array (float), L spectral points,
;		M observations, N detectors
;	freq_scale = frequency scale (double array, L elements),
;		     which is dindgen(L)*delta_frequency+start_frequency
;
; KEYWORD OUTPUT:
;	start_frequency = attribute enumerating the start of the
;			  frequency grid for the spectra (double)
;	delta_frequency = attribute enumerating the grid spacing
;			  for the frequency grid of the spectra (double)
;	status = 0 for success
;		 1 if file does not exist
;		 2 the file name does not have correct version strings in name
;		 3 the file is not HDF 5
; 
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 07/2005
;
; UPDATE HISTORY:
;
;-



pro read_l1b_hdf_spectra,file,filter,spectra,freq_scale,status=status, $
  start_frequency = start_frequency, delta_frequency = delta_frequency

if n_params() ne 4 then begin
    print,'Usage:'
    print,'read_l1b_hdf_spectra,file,filter,spectra,freq_scale, $'
    print,'status=status,start_frequency = start_frequency, $'
    print,'delta_frequency = delta_frequency'
    return
endif

status=0
; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return
endif
; check version
pos2 = strpos(file,'_F02_')
pos3 = strpos(file,'_F03_')
if pos2 ne -1 then version = 2
if pos3 ne -1 then version = 3
pos=0
if pos eq -1 then begin
    print,'Expected version _F02_ or _F03_ in file name not found.'
    print,file
    status = 2
    return
endif
; check to see if file is an HDF5 
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return
endif


fid = h5f_open(file)

groupid = h5g_open(fid,'Filter'+strupcase(filter))

specid = h5d_open(groupid,'Spectra')
spectra = h5d_read(specid)
h5d_close,specid

freqid = h5a_open_name(groupid,'Start_Frequency')
start_frequency = h5a_read(freqid)

delnuid = h5a_open_name(groupid,'Delta_Frequency')
delta_frequency = h5a_read(delnuid)

h5a_close,freqid
h5a_close,delnuid

h5g_close, groupid
h5f_close,fid

; make frequency scale
s = size(spectra)
freq_scale=dindgen(s[1])*delta_frequency+start_frequency



return

end
